/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.tasks;

import java.awt.Window;
import java.util.Properties;

import org.cytoscape.application.swing.CytoPanel;
import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.application.swing.CytoPanelState;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.core.NetworkParser;
import de.hhu.ba.yoshikoWrapper.core.ParameterSet;
import de.hhu.ba.yoshikoWrapper.core.StatusInformer;
import de.hhu.ba.yoshikoWrapper.cytoUtil.NodeMap;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoCluster;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoResult;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoSolution;
import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.ClusterEditingSolutions;
import de.hhu.ba.yoshikoWrapper.swig.CoreAlgorithm;
import de.hhu.ba.yoshikoWrapper.swig.IntVector;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInput;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;
import de.hhu.ba.yoshikoWrapper.swing.components.ResultPanel;


public class AlgorithmTask extends AbstractTask {

	private String SOLUTION_COLUMN_PREFIX = "yoshikoSolution_";

	//Symbolic links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();
	private Window statusWindow;

	//Parameters
	private final ParameterSet parameterSet;
	private final CyNetwork net;

	//Temps, need to be freed in C++
	private LibraryInput input;
	private ClusterEditingSolutions result;
	private CoreAlgorithm ca;

	private ResultPanel resultPanel;

	public AlgorithmTask(// <<< Too many variables here, some should be grouped later TODO:
			Window statusWindow,
			CyNetwork net,
			ParameterSet parameterSet
			)
	{
		this.statusWindow = statusWindow;
		this.net = net;
		this.parameterSet = parameterSet;
	}

	@Override
	public void run(TaskMonitor taskMonitor) throws Exception {


		taskMonitor.setTitle(LocalizationManager.get("yoshTask"));
		taskMonitor.setProgress(0.0);

		//Get current network
		if (net == null) {
			logger.warn("CoreAlgorithm called on a net that is NULL!");
			throw new Exception("CoreAlgorithm called on a net that is NULL!");
		}

		//Set time limit
		LibraryInterface.setTimeLimit(parameterSet.timeLimit);
		if (parameterSet.disableMultiThreading) {
			LibraryInterface.setThreadLimit(1);
		}
		else {
			LibraryInterface.setThreadLimit(0);
		}

		//Create a node map to identify nodes and eges in the solution
		NodeMap nodeMap  = new NodeMap(net);
		taskMonitor.setProgress(0.1);

		//Generate an input instance from the network
		input = NetworkParser.parseNetwork(
				net,
				nodeMap,
				parameterSet.weightColumn,
				parameterSet.permanentColumn,
				parameterSet.forbiddenColumn,
				parameterSet.defaultDeletionCost
				);
		taskMonitor.setProgress(0.2);

		//Set the default value for insertion cost
		input.setDefaultInsertionCost(parameterSet.defaultInsertionCost);

		//Call Yoshiko <<< Algorithm is performed here
		ca = LibraryInterface.getRun(input,
				parameterSet.solCount,
				parameterSet.reductionRulesBitMask,
				parameterSet.snrMultFactor,
				parameterSet.usePartitionCuts,
				parameterSet.useTriangleCuts,
				parameterSet.useHeuristic
		);


		ca.registerCplexInformer(new StatusInformer(statusWindow,taskMonitor));

		result  = ca.run();

		taskMonitor.setProgress(0.9);

		checkForInvalidSolutions(result);

		long numberOfSolutions = result.getNumberOfSolutions();

		taskMonitor.setStatusMessage("Found: "+numberOfSolutions+" solutions!"); //TODO localize

		YoshikoResult yoshikoResult = new YoshikoResult();
		yoshikoResult.flags = result.getFlags();


		//Loop over (multiple) solutions
		for (long i=0;i<numberOfSolutions;i++) {

			taskMonitor.setStatusMessage("Processing solution "+(i+1)+" of "+numberOfSolutions);
			YoshikoSolution solution = new YoshikoSolution(net,i);
			String columnName = SOLUTION_COLUMN_PREFIX+(i+1);

			net.getDefaultNodeTable().deleteColumn(columnName);
			net.getDefaultNodeTable().createColumn(columnName, String.class, false);

			//Fetch number of clusters in the solution
			long numberOfClusters = result.getNumberOfClusters(i);

			//Loop over cluster
			for (long k=0;k<numberOfClusters;k++) {
				//Create java instance
				YoshikoCluster cluster = new YoshikoCluster(k,net);


				taskMonitor.setStatusMessage("Processing cluster "+(k+1)+" of "+numberOfClusters);

				IntVector clusterVector = result.getCluster(i, k);

				long sizeOfCluster = clusterVector.size();
				for (int l=0;l<sizeOfCluster;l++) { //Unsafe mismatch int long
					taskMonitor.setStatusMessage("Processing node "+(l+1)+ " of "+sizeOfCluster);
					int nodeID = clusterVector.get(l);
					CyNode node = nodeMap.indexOf(nodeID); //<<< Another int/long conversion
					cluster.addNode(node);

					net.getRow(node).set(columnName, ""+(k+1)); //Add Cluster ID in table (Remove in final version?)
				}
				cluster.createSubNetwork();
				//Register cluster with solution for further reference
				solution.cluster.add(cluster);
			}
			//Register solution with result for further reference
			yoshikoResult.solutions.add(solution);
		}

		taskMonitor.setProgress(0.99);
		//At this point the C++ objects can be freed, in case that the algorithm terminates beforehand they need to be cleaned-up elsewhere
		ca.delete();
		input.delete();
		result.delete();

		//Generate solutionsPanel
		resultPanel = new ResultPanel(yoshikoResult);

		//Show solution panel
		CyCore.registrar.registerService(resultPanel,CytoPanelComponent.class, new Properties());
		//Focus solution panel
		CytoPanel eastPanel = CyCore.swing.getCytoPanel(CytoPanelName.EAST);
		eastPanel.setSelectedIndex(eastPanel.indexOfComponent(resultPanel));
		//Show (might be invisible)
		eastPanel.setState(CytoPanelState.DOCK);

	}

	private void checkForInvalidSolutions(ClusterEditingSolutions result) throws Exception {
		if (result == null) {
			//There was no result object generated at all
			throw new Exception(LocalizationManager.get("noFeasible"));
		}

		if (result.getNumberOfSolutions() == 0) {
			//The result object is empty
			throw new Exception(LocalizationManager.get("noFeasible"));
		}
		if (result.getNumberOfSolutions() == 1) {
			//The result doesn't contain any clusters
			if (result.getNumberOfClusters(0) == 0) {
				throw new Exception(LocalizationManager.get("noFeasible"));
			}
		}

	}

	@Override
	public void cancel() {
		//Free C++ resources
		if (ca != null) {
			System.out.println("Deleting CoreAlgorithm instance!");
			ca.cancel();
			ca.delete();
		}
		if (input != null) {
			input.delete();
		}
		if (result != null) {
			result.delete();
		}
		//Delete (incomplete) result panel
		if (resultPanel != null) {
			resultPanel.deleteSolution();
		}
		super.cancel();
	}
}
